<?php
/**
 * Copyright 2011 pixeltricks GmbH
 *
 * This file is part of SilverCart.
 *
 * SilverCart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SilverCart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SilverCart.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @package SilvercartShipmentDhl
 * @subpackage Backend
 */

/**
 * A modified TableListField for the SilverCart DHL order export manager.
 *
 * @package SilvercartShipmentDhl
 * @subpackage Backend
 * @author Sascha Koehler <skoehler@pixeltricks.de>
 * @since 03.08.2011
 * @copyright 2011 pixeltricks GmbH
 * @license http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 */
class SilvercartShipmentDhlOrderTableListField extends SilvercartEditableTableListField {

    /**
     * The label for the export button
     *
     * @var string
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 03.08.2011
     */
    public $exportButtonLabel = 'Export to DHL';

    /**
     * Contains the path to the export directory.
     * 
     * @var string
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 04.08.2011
     */
    protected $exportDirectory;

    /**
     * Contains the configuration object.
     *
     * @var SilvercartShipmentDhl
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 03.08.2011
     */
    protected $silvercartShipmentDhl;

    /**
     * We set the paths to the export directory here.
     *
     * @param string $name         The name of the tableField
     * @param string $sourceClass  The source class
     * @param string $fieldList    The name of the method that provides the field list
     * @param string $sourceFilter The filter
     * @param string $sourceSort   A SQL sort statement
     * @param string $sourceJoin   A SQL join statement
     *
     * @return void
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 04.08.2011
     */
    public function __construct($name, $sourceClass, $fieldList = null, $sourceFilter = null, $sourceSort = null, $sourceJoin = null) {
        parent::__construct($name, $sourceClass, $fieldList, $sourceFilter, $sourceSort, $sourceJoin);
        
        $this->silvercartShipmentDhl = DataObject::get_one('SilvercartShipmentDhl');

        if ($this->silvercartShipmentDhl &&
            !empty($this->silvercartShipmentDhl->exportDirectory) &&
            is_dir($this->silvercartShipmentDhl->exportDirectory)) {

            $this->exportDirectory = $this->silvercartShipmentDhl->exportDirectory;
        } else {
            $this->exportDirectory = Director::baseFolder().'/silvercart_shipment_dhl/export_files/';
        }
        
        if (substr($this->exportDirectory, -1) != '/') {
            $this->exportDirectory .= '/';
        }
    }

    /**
     * We overwrite the export function here to avoid the download of the
     * generated file.
     *
     * @return void
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 04.08.2011
     */
    public function export() {
        $this->generateExportFileData();
        
        Director::redirectBack();
    }

    /**
     * This is the place where the export file is generated and the timestamp
     * is written to the database.
     *
     * @return void
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>, Sebastian Diel <sdiel@pixeltricks.de>
     * @since 21.09.2012
     */
    public function generateExportFileData() {
        if (!$this->sourceItems()) {
            return false;
        }
        
        $fileName = $this->getFileName();

        if ($fp = fopen($this->exportDirectory.$fileName, 'w')) {
            if ($this->silvercartShipmentDhl->activateCsvHeaders) {
                fwrite($fp, $this->getHeaderRow());
            }
            
            foreach ($this->sourceItems() as $order) {
                fwrite($fp, $this->getCsvRowFromRecord($order));
                if ($this->silvercartShipmentDhl->OrderStatusAfterExportID > 0 &&
                    $this->silvercartShipmentDhl->OrderStatusAfterExport() instanceof SilvercartOrderStatus) {
                    $order->SilvercartOrderStatusID = $this->silvercartShipmentDhl->OrderStatusAfterExportID;
                    $order->write();
                }
            }

            fclose($fp);
            
            $this->silvercartShipmentDhl->setField('lastExportDateTime', date('Y-m-d H:i:s'));
            $this->silvercartShipmentDhl->write();
        }
        
        return $fileName;
    }

    /**
     * Returns a unqiue filename for the export file.
     *
     * @return string
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 04.08.2011
     */
    protected function getFileName() {
        $fileName       = 'dhl_export_'.date('Y-m-d-H-i-s').'_';
        $extension      = '.csv';
        $fileCounter    = 0;

        while (file_exists($this->exportDirectory.$fileName.str_pad($fileCounter, 3, '0', STR_PAD_LEFT).$extension)) {
            $fileCounter++;
        }

        return $fileName.str_pad($fileCounter, 3, '0', STR_PAD_LEFT).$extension;
    }

    /**
     * Returns a string in CSV format from the given product's data.
     *
     * @param SilvercartOrder $orderObj The order to extract the data from
     * 
     * @return string
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 03.08.2011
     */
    protected function getCsvRowFromRecord(SilvercartOrder $orderObj) {
        $includeRow     = true;
        $rowElements    = array();
        $row            = '';
        $callbackClass  = $this->class.'_'.$this->name;

        if (class_exists($callbackClass)) {
            $callbackReflectionClass = new ReflectionClass($callbackClass);
        }

        $exportFields = $this->silvercartShipmentDhl->SilvercartShipmentDhlFields();
        $exportFields->sort('sortOrder', 'ASC');

        if (class_exists($callbackClass)) {
            if ($callbackReflectionClass->hasMethod('includeRow')) {
                // use eval instead of generic static method call to keep
                // compatible with PHP version < 5.3
                // eval is dirty, but it works for older versions...
                $includeRow = eval('return ' . $callbackClass . '::includeRow($orderObj);');
            }
        }

        if ($includeRow) {
            foreach ($exportFields as $exportField) {
                if ($exportField->isCallbackField) {
                    $fieldValue = '';
                } else {
                    $fieldObj = DataObject::get_one(
                        'SilvercartShipmentDhlField',
                        sprintf(
                            "SilvercartShipmentDhlID = %d AND name = '%s'",
                            $this->silvercartShipmentDhl->ID,
                            $exportField->name
                        )
                    );

                    if ($fieldObj) {
                        $fieldValue = $fieldObj->defaultValue;
                    }

                    if (empty($fieldValue)) {
                        $fieldValue = $this->getDhlFieldValueFor($exportField->name, $orderObj);
                    }
                }

                // If a callback class and method exist for this exporter and field
                // we use it's return value as field value.
                if (class_exists($callbackClass)) {
                    $callbackMethod = $exportField->name;
                    if ($callbackReflectionClass->hasMethod($callbackMethod)) {
                        // use eval instead of generic static method call to keep
                        // compatible with PHP version < 5.3
                        // eval is dirty, but it works for older versions...
                        //$fieldValue = $callbackClass::$callbackMethod($productObj, $fieldValue);
                        $fieldValue = eval('return ' . $callbackClass . '::' . $callbackMethod . '($orderObj, $fieldValue);');
                    }
                }

                $rowElements[]  = str_replace('"', '\'', $fieldValue);
            }
            $orderObj->setField('hasBeenExportedToDhl', true);
            $orderObj->write();
        }

        $row  = implode($this->getPreparedCsvSeparator(), $rowElements);
        $row .= "\n";

        return iconv('UTF8', 'LATIN1', $row);
    }

    /**
     * Returns a string in CSV format that is used as header.
     *
     * @return string
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 03.08.2011
     */
    protected function getHeaderRow() {
        $rowElements = array();
        $row         = '';

        $exportFields = $this->silvercartShipmentDhl->SilvercartShipmentDhlFields();
        $exportFields->sort('sortOrder', 'ASC');

        foreach ($exportFields as $exportField) {
            if (empty($exportField->headerTitle)) {
                $headerTitle = $exportField->name;
            } else {
                $headerTitle = $exportField->headerTitle;
            }
            $rowElements[] = $headerTitle;
        }

        $row  = implode($this->getPreparedCsvSeparator(), $rowElements);
        $row .= "\n";

        return $row;
    }

    /**
     * Returns a value from the given order object for the given field.
     *
     * @param string          $fieldName The name of the field to return the value for.
     * @param SilvercartOrder $order     The order object to retrieve the data from.
     *
     * @return string
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>, Sebastian Diel <sdiel@pixeltricks.de>
     * @since 10.10.2012
     */
    protected function getDhlFieldValueFor($fieldName, SilvercartOrder $order) {
        $fieldValue = '';

        switch ($fieldName) {
            case 'POOL_REFNR':
                $fieldValue = 'ORDER'.$order->OrderNumber;
                break;
            case 'POOL_EMPF_REFNR':
                $fieldValue = 'CSTER'.$order->Member()->CustomerNumber;
                break;
            case 'POOL_EMPF_NAME1':
                $fieldValue = $order->SilvercartShippingAddress()->FirstName.' '.$order->SilvercartShippingAddress()->Surname;
                break;
            case 'POOL_EMPF_NAME2':
                $fieldValue = $order->SilvercartShippingAddress()->PostNumber;
                break;
            case 'POOL_EMPF_PLZ':
                $fieldValue = $order->SilvercartShippingAddress()->Postcode;
                break;
            case 'POOL_EMPF_ORT':
                $fieldValue = $order->SilvercartShippingAddress()->City;
                break;
            case 'POOL_EMPF_STRASSE':
                if ($order->SilvercartShippingAddress()->IsPackstation) {
                    $fieldValue = $order->SilvercartShippingAddress()->Packstation;
                } else {
                    $fieldValue = $order->SilvercartShippingAddress()->Street;
                }
                break;
            case 'POOL_EMPF_HAUSNR':
                $fieldValue = $order->SilvercartShippingAddress()->StreetNumber;
                break;
            case 'POOL_EMPF_VORNAME':
                $fieldValue = $order->SilvercartShippingAddress()->FirstName;
                break;
            case 'POOL_EMPF_NACHNAME':
                $fieldValue = $order->SilvercartShippingAddress()->Surname;
                break;
            case 'POOL_EMPF_LANDCODE':
                $fieldValue = $order->SilvercartShippingAddress()->SilvercartCountry()->ISO2;
                break;
            case 'POOL_ABS_BEMERKUNG':
                $fieldValue = $order->SilvercartShippingAddress()->Addition;
                break;
            case 'POOL_EMPF_BEMERKUNG':
                $fieldValue = $order->SilvercartShippingAddress()->Addition;
                break;
            case 'POOL_GEWICHT':
                $weightInKg = $order->WeightTotal / 1000;
                $fieldValue = str_replace('.', ',', (string) $weightInKg);
                break;
        }

        return $fieldValue;
    }

    /**
     * Returns the separator for the CSV fields.
     *
     * @return string
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 03.08.2011
     */
    protected function getPreparedCsvSeparator() {
        $separator = $this->silvercartShipmentDhl->csvSeparator;

        if ($separator == '\t') {
            $separator = "\t";
        }

        return $separator;
    }
}
