<?php
/**
 * Copyright 2011 pixeltricks GmbH
 *
 * This file is part of SilverCart.
 *
 * SilverCart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SilverCart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SilverCart.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @package SilvercartShipmentDhl
 * @subpackage Base
 */

/**
 * The Silvercart DHL shipment import.
 *
 * @package SilvercartShipmentDhl
 * @subpackage Base
 * @author Sascha Koehler <skoehler@pixeltricks.de>
 * @copyright 2011 pixeltricks GmbH
 * @since 04.08.2011
 * @license http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 */
class SilvercartShipmentDhlImport extends DataObject {
    
    /**
     * Contains the path to the directory from where files from Easylog should
     * be loaded.
     * 
     * @var string
     */
    protected $importDirectory;
    
    /**
     * Attributes.
     *
     * @var array
     */
    public static $db = array(
        'SITEMS_ID'                 => 'Int',           // Interne ID der Sendung
        'SITEMS_IDENTCODE'          => 'VarChar(20)',   // Identcode
        'SITEMS_STORNO'             => 'Int',           // -1, Sendung storniert, 0, Sendung nicht storniert
        'SITEMS_DATE'               => 'Date',          // Versanddatum der Sendung
        'EmpKundenNr'               => 'VarChar(10)',   // Referenznummer des Empfängers
        'SITEMS_LEITCODE'           => 'VarChar(14)',   // Leitcode des Empfängers
        'SITEMS_ORDERNUMBER'        => 'VarChar(20)',   // Lieferscheinnummer des Sendung
        'SOWNER_CLOSEDATE'          => 'Date',          // Datum des Sendungsabschlusses
        'SITEMS_KOLLICOUNTER'       => 'Int',           // ID für Mehrpaketsendungen (Kolli, Multitrack)
        'SITEMS_STORNO_REASON'      => 'VarChar(80)',   // Stornierungsgrund
        'SITEMS_IDENTCODE_RETOUR'   => 'VarChar(26)',   // Identcode bei Retoure
        'TEILNAHME'                 => 'Int'            // Teilnahmenummer
    );
    
    /**
     * Sets the singular name for this object.
     *
     * @return string
     *
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 10.07.2012
     */
    public function singular_name() {
        return SilvercartTools::singular_name_for($this);
    }
    
    /**
     * Sets the plural name for this object.
     *
     * @return string
     *
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 10.07.2012
     */
    public function plural_name() {
        return SilvercartTools::plural_name_for($this);
    }
    
    /**
     * Field labels for display in tables.
     *
     * @param boolean $includerelations A boolean value to indicate if the labels returned include relation fields
     *
     * @return array
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 30.08.2011
     */
    public function fieldLabels($includerelations = true) {
        $fieldLabels = array_merge(
            parent::fieldLabels($includerelations),
            array(
                'SITEMS_IDENTCODE'      => _t('SilvercartShipmentDhlImport.SITEMS_IDENTCODE'),
                'SITEMS_DATE'           => _t('SilvercartShipmentDhlImport.SITEMS_DATE'),
                'EmpKundenNr'           => _t('SilvercartShipmentDhlImport.EmpKundenNr'),
                'SITEMS_ORDERNUMBER'    => _t('SilvercartShipmentDhlImport.SITEMS_ORDERNUMBER'),
                'TEILNAHME'             => _t('SilvercartShipmentDhlImport.TEILNAHME')
            )
        );

        $this->extend('updateFieldLabels', $fieldLabels);
        return $fieldLabels;
    }
    
    /**
     * Summaryfields for display in tables.
     *
     * @return array
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>, Sebastian Diel <sdiel@pixeltricks.de>
     * @since 10.07.2012
     */
    public function summaryFields() {
        $summaryFields = array(
            'SITEMS_IDENTCODE'      => $this->fieldLabel('SITEMS_IDENTCODE'),
            'SITEMS_DATE'           => $this->fieldLabel('SITEMS_DATE'),
            'EmpKundenNr'           => $this->fieldLabel('EmpKundenNr'),
            'SITEMS_ORDERNUMBER'    => $this->fieldLabel('SITEMS_ORDERNUMBER'),
            'TEILNAHME'             => $this->fieldLabel('TEILNAHME'),
        );
        
        $this->extend('updateSummaryFields', $summaryFields);
        return $summaryFields;
    }
    
    /**
     * We set the paths to the export directory here.
     *
     * @return void
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 04.08.2011
     */
    public function init() {
        parent::init();
        
        $this->silvercartShipmentDhl = DataObject::get_one('SilvercartShipmentDhl');

        if ($this->silvercartShipmentDhl &&
            !empty($this->silvercartShipmentDhl->importDirectory) &&
            is_dir($this->silvercartShipmentDhl->importDirectory)) {

            $this->importDirectory = $this->silvercartShipmentDhl->importDirectory;
        } else {
            $this->importDirectory = Director::baseFolder().'/silvercart_shipment_dhl/import_files/';
        }
    }
}

/**
 * The Silvercart DHL shipment import.
 *
 * @package SilvercartShipmentDhl
 * @subpackage Base
 * @author Sascha Koehler <skoehler@pixeltricks.de>
 * @copyright 2011 pixeltricks GmbH
 * @since 04.08.2011
 * @license http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 */
class SilvercartShipmentDhlImport_CollectionController extends ModelAdmin_CollectionController {
    
    /**
     * We don't want to show the import form here.
     * 
     * @var bool
     */
    public $showImportForm = false;
    
    /**
     * We don't want to show a create form here.
     *
     * @return boolean false
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 03.08.2011
     */
    public function CreateForm() {
        return false;
    }
}