<?php
/**
 * Copyright 2011 pixeltricks GmbH
 *
 * This file is part of SilverCart.
 *
 * SilverCart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SilverCart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SilverCart.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @package SilvercartShipmentDhl
 * @subpackage Orders
 */

/**
 * Injects new datafields into the original SilvercartOrder object.
 *
 * @package SilvercartShipmentDhl
 * @subpackage Orders
 * @copyright pixeltricks GmbH
 * @author Sascha Koehler <skoehler@pixeltricks.de>
 * @since 02.08.2011
 * @license http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 */
class SilvercartShipmentDhlOrder extends DataObjectDecorator {
    
    /**
     * Extends the database fields and relations of the decorated class.
     *
     * @return array
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 02.08.2011
     */
    public function extraStatics() {
        return array(
            'db' => array(
                'hasBeenExportedToDhl'  => 'Boolean(0)',
                'trackingCodeNVE'       => 'VarChar(20)'
            ),
            'summary_fields' => array(
                'hasBeenExportedToDhl' => 'hasBeenExportedToDhl'
            )
        );
    }
    
    /**
     * Adjust the summay fields here.
     *
     * @param array &$fields Fields to update
     *
     * @return void
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 03.08.2011
     */
    public function updateSummaryFields(&$fields) {
        $fields['hasBeenExportedToDhl'] = _t('SilvercartShipmentDhlOrder.HAS_BEEN_EXPORTED_TO_DHL');
    }
    
    /**
     * Set the field labels for our new fields here.
     *
     * @param array &$labels The field labels
     *
     * @return void
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 03.08.2011
     */
    public function updateFieldLabels(&$labels) {
        $labels['hasBeenExportedToDhl'] = _t('SilvercartShipmentDhlOrder.HAS_BEEN_EXPORTED_TO_DHL');
        $labels['trackingCodeNVE']      = _t('SilvercartShipmentDhlOrder.TRACKING_CODE_NVE');
    }
    
    /**
     * Add new fields to the searchable fields.
     *
     * @param array &$fields Field to update
     *
     * @return void
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 03.08.2011
     */
    public function updateSearchableFields(&$fields) {
        $fields['hasBeenExportedToDhl'] = array(
            'title'     =>  _t('SilvercartShipmentDhlOrder.HAS_BEEN_EXPORTED_TO_DHL'),
            'filter'    => 'ExactMatchFilter'
        );
    }
    
    /**
     * Updates an order default tracking code.
     * 
     * @param string &$trackingCode Tracking code to update.
     * 
     * @return void
     *
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 21.11.2013
     */
    public function updateTrackingCode(&$trackingCode) {
        if (!empty($this->owner->trackingCodeNVE)) {
            $trackingCode = $this->owner->trackingCodeNVE;
        }
    }
    
    /**
     * Updates an order default tracking link.
     * 
     * @param string &$trackingLink Tracking link to update.
     * 
     * @return void
     *
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 21.11.2013
     */
    public function updateTrackingLink(&$trackingLink) {
        if (!empty($this->owner->trackingCodeNVE)) {
            $trackingLink = $this->getTrackingLinkDHL();
        }
    }
    
    /**
     * Returns the DHL tracking link.
     * 
     * @return string
     */
    public function getTrackingLinkDHL() {
        $trackingLink = '';
        if (!empty($this->owner->trackingCodeNVE)) {
            $shipmentDhl = DataObject::get_one('SilvercartShipmentDhl');
            if ($shipmentDhl) {
                $trackingLink = $shipmentDhl->trackingUrl . $this->owner->trackingCodeNVE;
            }
        }
        return $trackingLink;
    }
}

/**
 * Injects new datafields into the original SilvercartOrder object.
 *
 * @package SilvercartShipmentDhl
 * @subpackage Orders
 * @copyright pixeltricks GmbH
 * @author Sascha Koehler <skoehler@pixeltricks.de>
 * @since 03.08.2011
 * @license http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 */
class SilvercartShipmentDhlOrder_CollectionController extends ModelAdmin_CollectionController {
    
    /**
     * We don't want to show the import form here.
     */
    public $showImportForm = false;
    
    /**
     * We don't want to show a create form here.
     *
     * @return boolean false
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 03.08.2011
     */
    public function CreateForm() {
        return false;
    }
    
    /**
     * Remove surplus fields here.
     *
     * @return Form
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 03.08.2011
     */
    public function SearchForm() {
        $form = parent::SearchForm();
        
        $form->Fields()->removeByName('hasBeenExported');
        
        return $form;
    }
    
    /**
     * We adjust the summary columns for the order result set here.
     * 
     * @return array
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 03.08.2011
     */
    public function columnsAvailable() {
        $columns = singleton($this->modelClass)->summaryFields();
        
        unset($columns['hasBeenExported']);
        
        return $columns;
    }
}

/**
 * Injects new datafields into the original SilvercartOrder object.
 *
 * @package SilvercartShipmentDhl
 * @subpackage Orders
 * @copyright pixeltricks GmbH
 * @author Sebastian Diel <sdiel@pixeltricks.de>
 * @since 10.07.2012
 * @license http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 */
class SilvercartShipmentDhlOrderDecorator_CollectionController extends DataObjectDecorator {
    
    
    /**
     * Extended allowed actions for the decorated collection controller
     *
     * @var type 
     */
    public static $allowed_actions = array(
        'DownloadDhlExport',
    );

    /**
     * Updates the results table
     *
     * @param SilvercartEditableTableListField $tableField      Field to update
     * @param array                            &$searchCriteria Search criteria
     * 
     * @return void
     *
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 10.07.2012
     */
    public function getResultsTable($tableField, &$searchCriteria) {
        $tableField->addBatchAction(
                array(
                    'action'    => 'exportDhlEasylog',
                    'label'     => _t('SilvercartShipmentDhlOrder.EXPORT_DHL_EASYLOG'),
                )
        );
    }
    
    /**
     * Batch action to export Dhl Easylog data
     *
     * @param array $orderIDs Order IDs to get invoice collection for
     * 
     * @return string
     *
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 11.07.2012 
     */
    public function silvercartBatch_exportDhlEasylog($orderIDs) {
        if (is_array($orderIDs) &&
            count($orderIDs) > 0) {
            $exportTable = new SilvercartShipmentDhlOrderTableListField(
                    'SilvercartOrder',
                    'SilvercartOrder',
                    null,
                    sprintf(
                            "`SilvercartOrder`.`ID` IN (%s)",
                            implode(',', $orderIDs)
                    )
            );
            $dhlEasylogFilePath = $exportTable->generateExportFileData();
            foreach ($orderIDs as $orderID) {
                $order = DataObject::get_by_id('SilvercartOrder', $orderID);
                if ($order) {
                    $order->markAsSeen();
                }
            }
            Session::set('SilvercartShipmentDhlOrder.DhlEasylogFileName', $dhlEasylogFilePath);
            Session::save();
            return 'document.location.href = "/admin/silvercart-orders/SilvercartOrder/DownloadDhlExport";';
        }
    }
    
    /**
     * Triggers the download for a batch generated Dhl Easylog export
     *
     * @return void
     *
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 11.07.2012 
     */
    public function DownloadDhlExport() {
        $exportDirectory    = SilvercartShipmentDhl::getExportDirectory();
        $filename           = Session::get('SilvercartShipmentDhlOrder.DhlEasylogFileName');
        if (file_exists($exportDirectory . $filename)) {
            header('Content-Type: text/csv');
            header('Content-Disposition: attachment; filename="dhl_easylog_export_' . date('Y-m-d_H-i-s') . '.csv"');
            print file_get_contents($exportDirectory . $filename);
            exit();
        } else {
            Director::redirectBack();
        }
    }
}